#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>		/* normal stuff             */
#include <ctype.h>
#include <sys/utsname.h>
#include <sys/times.h>

/* ensure getopt */
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

/* ensure sys/types */
#ifndef _SYS_TYPES_H
#include <sys/types.h>
#endif

/* some systems need this */
#ifdef HAVE_MATH_H
#include <math.h>
#endif

/* SunOS 4.x Fix */
#ifndef CLK_TCK
#define CLK_TCK _SC_CLK_TCK
#endif

#include "parser.h"
#include "linklist.h"
#include "hashtab.h"
#include "seoanalyzer.h"
#include "botvisits.h"
#include "monthlydata.h"
#include "pagerank.h"

SNODEPTR *s_array = NULL;	/* search strings           */
SNODEPTR *k_array = NULL;	/* search strings           */
PNODEPTR *p_array = NULL;	/* search strings           */
int s_count;
int k_count;
int p_count;

#define MIN(x,y)  ( ((x)>(y)) ? (y) : (x) )


int keywords_per_page = 20;
int pages_per_page = 20;
extern char *hostname;

#define msg_nomem_tsr "noemm"

void output_style ();
void top_search_table (SNODEPTR * s_array, int s_count, int t_val);
void write_index_html ();





FILE *out_fp;

/*********************************************/
/* QS_SRCH_CMPH - QSort cmp srch str by hits */
/*********************************************/

int
qs_srch_cmph (const void *cp1, const void *cp2)
{
  u_long t1, t2;
  t1 = (*(SNODEPTR *) cp1)->count;
  t2 = (*(SNODEPTR *) cp2)->count;
  if (t1 != t2)
    return (t2 < t1) ? -1 : 1;
  /* if hits are the same, we sort by search string instead */
  return strcmp ((*(SNODEPTR *) cp1)->string, (*(SNODEPTR *) cp2)->string);
}

int
qs_page_cmph (const void *cp1, const void *cp2)
{
  u_long t1, t2;
  t1 = (*(PNODEPTR *) cp1)->count;
  t2 = (*(PNODEPTR *) cp2)->count;
  if (t1 != t2)
    return (t2 < t1) ? -1 : 1;
  /* if hits are the same, we sort by search string instead */
  return strcmp ((*(PNODEPTR *) cp1)->pageurl, (*(PNODEPTR *) cp2)->pageurl);
}



void
write_month_html ()
{
  char filename[0x100];
  int item;
  int total_val;
  int cnt;
  SNODEPTR *ptr;


  /* do search string related stuff, sorting appropriately...              */
  if ((s_count = load_srch_array (sr_htab, NULL)))
    {
      if ((s_array = malloc (sizeof (SNODEPTR) * (s_count))) != NULL)
	{
	  s_count = load_srch_array (sr_htab, s_array);
	  monthlydata->phrasecount = s_count;

#if 0
	  if (ntop_search || dump_search)
	    {
#endif
	      qsort (s_array, s_count, sizeof (SNODEPTR), qs_srch_cmph);
#if 0
	      if (ntop_search)
		top_search_table ();	/* top search strings table     */
	      if (dump_search)
		dump_all_search ();	/* dump search string tab file  */
	    }
#endif

	  /* Count total value */
	  cnt = s_count;
	  ptr = s_array;
	  total_val = 0;
	  while (cnt--)
	    {
	      total_val += (u_long) ((SNODEPTR) (*ptr)->count);
	      ptr++;
	    }


	  for (item = 0;
	       item < (s_count + keywords_per_page - 1) / keywords_per_page;
	       item++)
	    {
	      if (item == 0)
		sprintf (filename, "searchstrings_%04d_%02d.html", cur_year,
			 cur_month);
	      else
		sprintf (filename, "searchstrings_%04d_%02d_%d.html",
			 cur_year, cur_month, item);
	      out_fp = fopen (filename, "w");
	      output_style (out_fp);

            fprintf (out_fp,
	      "<div> <table border=0 cellspacing=0 cellpadding=0 align=center>");

            fprintf (out_fp,"<tr><td align=left colspan=5>");

	      if (item != 0)
		if (item == 1)
		  fprintf (out_fp,
			   "<a href=searchstrings_%04d_%02d.html>&lt;&lt; Previous</a>",
			   cur_year, cur_month);
		else
		  fprintf (out_fp,
			   "<a href=searchstrings_%04d_%02d_%d.html>&lt;&lt; Previous</a> ",
			   cur_year, cur_month, item - 1);

            fprintf (out_fp,"</td><td align=right colspan=4>");

	      if (((item + 1) * keywords_per_page) < s_count)
		fprintf (out_fp,
			 "<a href=searchstrings_%04d_%02d_%d.html>Next &gt;&gt;</a>",
			 cur_year, cur_month, item + 1);
            fprintf (out_fp,"</td></tr>\n");

	      top_search_table (&s_array[item * keywords_per_page], MIN (keywords_per_page, s_count - item * keywords_per_page), total_val);	/* top keywords page */

	      page_footer (out_fp);
	      fclose (out_fp);
	    }


	  free (s_array);
	}
      else if (verbose)
	fprintf (stderr, "%s [s_array]\n", msg_nomem_tsr);	/* err */
    }

  /* do search string related stuff, sorting appropriately...              */
  if ((k_count = load_srch_array (kw_htab, NULL)))
    {
      if ((k_array = malloc (sizeof (SNODEPTR) * (k_count))) != NULL)
	{
	  k_count = load_srch_array (kw_htab, k_array);
	  monthlydata->keywordcount = k_count;

#if 0
	  if (ntop_search || dump_search)
	    {
#endif
	      qsort (k_array, k_count, sizeof (SNODEPTR), qs_srch_cmph);
#if 0
	      if (ntop_search)
		top_search_table ();	/* top search strings table     */
	      if (dump_search)
		dump_all_search ();	/* dump search string tab file  */
	    }
#endif

	  /* Count total value */
	  cnt = k_count;
	  ptr = k_array;
	  total_val = 0;
	  while (cnt--)
	    {
	      total_val += (u_long) ((SNODEPTR) (*ptr)->count);
	      ptr++;
	    }



	  for (item = 0;
	       item < (k_count + keywords_per_page - 1) / keywords_per_page;
	       item++)
	    {
	      if (item == 0)
		sprintf (filename, "keywords_%04d_%02d.html", cur_year,
			 cur_month);
	      else
		sprintf (filename, "keywords_%04d_%02d_%d.html", cur_year,
			 cur_month, item);
	      out_fp = fopen (filename, "w");
	      output_style (out_fp);

            fprintf (out_fp,
	      "<div> <table border=0 cellspacing=0 cellpadding=0 align=center>");

            fprintf (out_fp,"<tr><td align=left colspan=5>");

	      if (item != 0)
		if (item == 1)
		  fprintf (out_fp,
			   "<a href=keywords_%04d_%02d.html>Previous</a>",
			   cur_year, cur_month);
		else
		  fprintf (out_fp,
			   "<a href=keywords_%04d_%02d_%d.html>Previous</a> ",
			   cur_year, cur_month, item - 1);

            fprintf (out_fp,"</td><td align=right colspan=4>");

	      if ((item + 1) * keywords_per_page < k_count)
		fprintf (out_fp,
			 "<a href=keywords_%04d_%02d_%d.html>Next</a>",
			 cur_year, cur_month, item + 1);


            fprintf (out_fp,"</td></tr>\n");


	      top_search_table (&k_array[item * keywords_per_page], MIN (keywords_per_page, k_count - item * keywords_per_page), total_val);	/* top keywords page */

	      page_footer (out_fp);
	      fclose (out_fp);
	    }

	  free (k_array);
	}
      else if (verbose)
	fprintf (stderr, "%s [k_array]\n", msg_nomem_tsr);	/* err */
    }





  /* do page related stuff, sorting appropriately...              */
  if ((p_count = load_srch_array (pg_htab, NULL)))
    {
      if ((p_array = malloc (sizeof (PNODEPTR) * (p_count))) != NULL)
	{
	  p_count = load_srch_array (pg_htab, p_array);
	  monthlydata->pagescount = p_count;

	      qsort (p_array, p_count, sizeof (PNODEPTR), qs_page_cmph);

	  /* Count total value */
	  cnt = p_count;
	  ptr = p_array;
	  total_val = 0;
	  while (cnt--)
	    {
	      total_val += (u_long) ((PNODEPTR) (*ptr)->count);
	      ptr++;
	    }



	  for (item = 0;
	       item < (p_count + pages_per_page - 1) / pages_per_page;
	       item++)
	    {
	      if (item == 0)
		sprintf (filename, "pages_%04d_%02d.html", cur_year,
			 cur_month);
	      else
		sprintf (filename, "pages_%04d_%02d_%d.html", cur_year,
			 cur_month, item);
	      out_fp = fopen (filename, "w");
	      output_style (out_fp);

            fprintf (out_fp,
	      "<div> <table border=0 cellspacing=0 cellpadding=0 align=center>");

            fprintf (out_fp,"<tr><td align=left colspan=6>");

	      if (item != 0)
		if (item == 1)
		  fprintf (out_fp,
			   "<a href=pages_%04d_%02d.html>Previous</a>",
			   cur_year, cur_month);
		else
		  fprintf (out_fp,
			   "<a href=pages_%04d_%02d_%d.html>Previous</a> ",
			   cur_year, cur_month, item - 1);


            fprintf (out_fp,"</td><td align=right colspan=5>");

	      if ((item + 1) * pages_per_page < p_count)
		fprintf (out_fp,
			 "<a href=pages_%04d_%02d_%d.html>Next</a>",
			 cur_year, cur_month, item + 1);

            fprintf (out_fp,"</td></tr>\n");

	      top_pages_table (&p_array[item * pages_per_page], MIN (pages_per_page, p_count - item * pages_per_page), total_val);	/* top keywords page */

	      page_footer (out_fp);
	      fclose (out_fp);
	    }

	  free (p_array);
	}
      else if (verbose)
	fprintf (stderr, "%s [p_array]\n", msg_nomem_tsr);	/* err */
    }




  sprintf (filename, "botvisits_%04d_%02d.html", cur_year, cur_month);
  out_fp = fopen (filename, "w");

  output_style ();
  output_bot_visits ();

  page_footer (out_fp);
  fclose (out_fp);
}

void
top_search_table (SNODEPTR * s_array, int s_count, int t_val)
{
  u_long cnt, tot_num;
  int i;
  int pagecount, pagecount_out;
  SNODEPTR sptr, *pointer;
  PNODEPTR *pages_array;
  int idcounter;
  int pos;

#if 0
  if ((t_ref == 0) || (count == 0))
    return;			/* don't bother if none to do    */
#endif

  cnt = tot_num = s_count;

#if 0
  if (tot_num > ntop_search)
    tot_num = ntop_search;
#endif


  idcounter = 0;
  fprintf (out_fp, "<h1>Top keywords SEOlizer website Statistics</h1>");
  fprintf (out_fp,
	   "<tr class=llist><td class=ltitle>&nbsp;</td><td class=ltitle>#</td>"
	   "<td colspan=2 class=ltitle>Hits</td><td class=ltitle>Search string</td><td class=ltitle>Pages</td>"
	   "<td colspan=2 class=ltitle>Page hits</td><td class=ltitle>URLs</td>");

  pointer = s_array;
  pos = 0;
  while (tot_num)
    {
      sptr = *pointer++;
      pagecount = load_page_array (sptr->pages, NULL);
      pagecount_out = pagecount > 10 ? 10 : pagecount;

      fprintf (out_fp, "<TR>\n");
      if (pagecount <= 1)
	fprintf (out_fp, "<td class=lrow>&nbsp;</td>\n");
      else
	{
	  fprintf (out_fp,
		   "<td class=\"lexpand lrow\"><a id=\"butt%d\" href=\"javascript:js_toggle(%d,%d,\'butt%d\')\">+</a></td>\n",
		   idcounter, idcounter + 2, idcounter + pagecount_out,
		   idcounter);
	  idcounter++;
	}


      fprintf (out_fp,
	       "<TD ALIGN=center class=lrow><FONT SIZE=\"-1\"><B>%d</B></FONT></TD>\n"
	       "<TD ALIGN=right class=lrow><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n"
	       "<TD ALIGN=right class=lrow><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n"
	       "<TD ALIGN=left NOWRAP class=lrow><FONT SIZE=\"-1\">",
	       pos + 1, sptr->count,
	       (t_val == 0) ? 0 : ((float) sptr->count / t_val) * 100.0);
      fprintf (out_fp, "%s", sptr->string);

	/* If we have hostname, output link to rankcheck! */
      if (hostname!=NULL) 
         fprintf (out_fp, "<a href=\"http://www.rankcheck.info/searchranking/googlekeywords.php?domain1=%s&keyword=%s\">&gt;&gt;</a>", hostname, sptr->string);

		fprintf(out_fp, "</font></td>\n");


      /* do search string related stuff, sorting appropriately...              */
      if (pagecount != 0)
	{
	  if ((pages_array =
	       malloc (sizeof (PNODEPTR) * (pagecount))) != NULL)
	    {
	      pagecount = load_page_array (sptr->pages, pages_array);
	      qsort (pages_array, pagecount, sizeof (PNODEPTR), qs_page_cmph);

	      for (i = 0; i < pagecount_out; i++)
		{
		  if (i == 0)
		    fprintf (out_fp, "<td class=lrow>%d</td>", pagecount);
		  else
		    fprintf (out_fp,
			     "</tr><tr id=\"row%d\"style=\"display: none\"><td colspan=6></td>",
			     idcounter);


		  fprintf (out_fp,
			   "<td class=lrow align=right>%d</td><td align=right class=lrow>%3.02f%%</td><td class=lrow>%s</td>\n",
			   pages_array[i]->count,
			   (t_val ==
			    0) ? 0 : ((float) pages_array[i]->count /
				      sptr->count) * 100.0,
			   pages_array[i]->pageurl);
		  idcounter++;
		}
	    }
	}

      fprintf (out_fp, "</tr>\n");


      tot_num--;
      pos++;
    }
  fprintf (out_fp, "<TR><TH HEIGHT=4></TH></TR>\n");


#if 0
  if ((all_search) && (a_ctr > ntop_search))
    {
      if (all_search_page (a_ctr, t_val))
	{
	  fprintf (out_fp, "<TR BGCOLOR=\"%s\">", GRPCOLOR);
	  fprintf (out_fp, "<TD COLSPAN=4 ALIGN=\"center\">\n");
	  fprintf (out_fp, "<FONT SIZE=\"-1\">");
	  fprintf (out_fp, "<A HREF=\"./search_%04d%02d.%s\">",
		   cur_year, cur_month, html_ext);
	  fprintf (out_fp, "%s</A></TD></TR>\n", msg_v_search);
	}
    }
#endif
  fprintf (out_fp, "</TABLE>\n</div>\n");
}

void
top_pages_table (PNODEPTR * p_array, int p_count, int t_val)
{
  u_long cnt, tot_num;
  int i;
  int searchcount, searchcount_out;
  PNODEPTR sptr, *pointer;
  SNODEPTR *searches_array;
  int idcounter;
  int pos;
  int pagerank;
  char fullurl[0x100];

  cnt = tot_num = p_count;

  fprintf (out_fp, "<h1>Top pages SEOlizer website Statistics</h1>");

  idcounter = 0;
  fprintf (out_fp,
	   "<tr class=llist><td class=ltitle>&nbsp;</td><td class=ltitle>#</td>"
	   "<td colspan=2 class=ltitle>Hits</td><td class=ltitle>Page</td>"
           "<td colspan=2 class=ltitle>PageRank</td>"
           "<td class=ltitle>Phrases</td>"
	   "<td colspan=2 class=ltitle>Search hits</td><td class=ltitle>Search Strings</td>");

  pointer = p_array;
  pos = 0;
  while (tot_num)
    {
      sptr = *pointer++;
      searchcount = load_page_array (sptr->searches, NULL);
      searchcount_out = searchcount > 20 ? 10 : searchcount;


      fprintf (out_fp, "<TR>\n");
      if (searchcount <= 1)
	fprintf (out_fp, "<td class=lrow>&nbsp;</td>\n");
      else
	{
	  fprintf (out_fp,
		   "<td class=\"lexpand lrow\"><a id=\"butt%d\" href=\"javascript:js_toggle(%d,%d,\'butt%d\')\">+</a></td>\n",
		   idcounter, idcounter + 2, idcounter + searchcount_out,
		   idcounter);
	  idcounter++;
	}


      fprintf (out_fp,
	       "<TD ALIGN=center class=lrow><FONT SIZE=\"-1\"><B>%d</B></FONT></TD>\n"
	       "<TD ALIGN=right class=lrow><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n"
	       "<TD ALIGN=right class=lrow><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n"
	       "<TD ALIGN=left NOWRAP class=lrow><FONT SIZE=\"-1\">",
	       pos + 1, sptr->count,
	       (t_val == 0) ? 0 : ((float) sptr->count / t_val) * 100.0);
      if (hostname!=NULL) 
         fprintf (out_fp, "<a href=\"http://%s%s\" rel=\"nofollow\">%s</a>", hostname, sptr->pageurl, sptr->pageurl);
         else fprintf (out_fp, "%s", sptr->pageurl);

      fprintf (out_fp, "</FONT></TD>\n");

      if (hostname!=NULL)  {
        strcpy(fullurl, "http://");
        strcat(fullurl, hostname);
        strcat(fullurl, sptr->pageurl);
        pagerank = getpagerank(fullurl);
        if (pagerank != -1) {
          fprintf (out_fp, "<td class=lrow>%d</td>\n", pagerank);
          fprintf (out_fp, "<td class=lrow><table cellspacing=0 cellpadding=0 border=0><tr>"
                  "<td bgcolor=\"#73AD53\" width=%d height=6><img width=%d height=6></td>"
                  "<td bgcolor=\"#CBD6DC\" width=%d height=6><img width=%d height=6></td>"
                  "</tr></table></td>\n", 
	           pagerank*6, pagerank*6, (10-pagerank)*6, (10-pagerank)*6);
         } else {
           fprintf (out_fp, "<td class=lrow>?</td><td class=lrow></td>\n");
         }
        } else
      fprintf (out_fp, "<td class=\"lrow\"></td><td class=\"lrow\"></td>\n");
        
      /* do search string related stuff, sorting appropriately...              */
      if (searchcount != 0)
	{
	  if ((searches_array =
	       malloc (sizeof (PNODEPTR) * (searchcount))) != NULL)
	    {
	      searchcount = load_page_array (sptr->searches, searches_array);
	      qsort (searches_array, searchcount, sizeof (SNODEPTR), qs_srch_cmph);

	      for (i = 0; i < searchcount_out; i++)
		{
		  if (i == 0)
		    fprintf (out_fp, "<td class=lrow>%d</td>", searchcount);
		  else
		    fprintf (out_fp,
			     "</tr><tr id=\"row%d\"style=\"display: none\"><td colspan=8></td>",
			     idcounter);


		  fprintf (out_fp,
			   "<td class=lrow align=right>%d</td><td align=right class=lrow>%3.02f%%</td><td class=lrow>%s",
			   searches_array[i]->count,
			   (t_val ==
			    0) ? 0 : ((float) searches_array[i]->count /
				      sptr->count) * 100.0,
			   searches_array[i]->string);

	/* If we have hostname, output link to rankcheck! */
      if (hostname!=NULL) 
         fprintf (out_fp, "<a href=\"http://www.rankcheck.info/searchranking/googlekeywords.php?domain1=%s&keyword=%s\">&gt;&gt;</a>", hostname, searches_array[i]->string);

		fprintf(out_fp, "</td>\n");

		  idcounter++;
		}
	    }
	}

      fprintf (out_fp, "</tr>\n");

      tot_num--;
      pos++;
    }
  fprintf (out_fp, "<TR><TH HEIGHT=4></TH></TR>\n");


  fprintf (out_fp, "</TABLE>\n</div>\n");
}


void
output_bot_visits ()
{
  int day;
  int crawlertotal;


  struct botlist *b;

  fprintf (out_fp, "<h1>Bot visits for %02d / %04d</h1>", cur_month,
	   cur_year);

  fprintf (out_fp, "<table border=0 cellspacing=0 cellpadding=0>");

  fprintf (out_fp,
	   "<tr><td class=ltitle>Crawler</td><td class=ltitle>Total</td>");
  for (day = 0; day < 31; day++)
    {
      fprintf (out_fp, "<td class=\"ltitle\"><b>%02d</b></td>\n", day + 1);
    }
  fprintf (out_fp, "</tr>");

  b = bot_visits;
  while (b != NULL)
    {
      fprintf (out_fp, "<tr><td class=ltitle>%s</td>", b->name);
      crawlertotal = 0;
      for (day = 0; day < 31; day++)
	crawlertotal += b->daycounts[day];
      fprintf (out_fp, "<td class=lrow align=right>%d</td>", crawlertotal);

      monthlydata->crawlerhits += crawlertotal;

      for (day = 0; day < 31; day++)
	{
	  fprintf (out_fp, "<td align=right class=lrow>%d</td>\n",
		   b->daycounts[day]);
	}
      fprintf (out_fp, "</tr>");

      b = b->next;
    }


  fprintf (out_fp, "</TABLE>\n<P>\n");
}


void
output_style ()
{
  fprintf (out_fp, "<html><head>");
  fprintf (out_fp, "<style><!--\n");
  fprintf (out_fp, "body { background-color: #c0c0f0;} \n");
  fprintf (out_fp, ".llist { min-width: 700px; } \n");
  fprintf (out_fp,
	   ".lexpand { background-color: #808080; width: 12px; text-align: center;} \n");
  fprintf (out_fp,
	   ".lexpand a { text-decoration: none; color: black; font-weight: bold;} \n");
  fprintf (out_fp,
	   ".ltitle { padding: 1px; background-color: rgb(209,211,212); border-top: 1px solid white; border-right: 1px solid #808080; border-left: 1px solid white; border-bottom: 1px solid #808080; text-align: center; }\n");
  fprintf (out_fp,
	   ".lrow { padding: 2px; background-color: white; border-bottom: 1px solid black; border-right: 1px solid #808080; } \n");
  fprintf (out_fp, "</style>\n");
  fprintf (out_fp, "</head><body>");

  fprintf (out_fp, ""
"<script language=\"javascript\">"
"function js_toggle( startid, endid, buttonid)"
"{"
"     for (var rowid=startid; rowid<=endid; rowid++) {"
"        var  name = 'row' + rowid;"
"      if (document.getElementById(name).style.display==\'none\'){"
"         document.getElementById(name).style.display = \'\';"
"         document.getElementById(buttonid).innerHTML=\'&nbsp;-&nbsp;\';"
"      } else {"
"         document.getElementById(name).style.display = \'none\';"
"         document.getElementById(buttonid).innerHTML=\'+\';"
"     }"
"     }"
"}"
"</script>");
}

void
page_footer ()
{
  char stime[0x100];
  struct tm *tm;
  time_t t;

  t = time(NULL);
  tm = localtime(&t);
  strftime(stime, sizeof(stime),"%Y-%B-%d %T (%z)",tm);

  fprintf(out_fp, "<hr>");
  fprintf (out_fp,
	   "Output generated by <a href=\"http://martin.hinner.info/seolizer/\">SEOlizer</a> version 0.2 at %s<br>", stime);
  fprintf (out_fp,
	   "Copyright (c) 2007 <a href=\"http://martin.hinner.info/\">Martin Hinner</a>. Sponsored by <a href=\"http://www.secons.com/\">SECONS Ltd.</a><br>");
}


void
write_index_html ()
{
  struct monthlydata *m;

  out_fp = fopen ("index.html", "w");

  output_style ();

  fprintf (out_fp, "<h1>SEOlizer website Statistics</h1>");

  fprintf (out_fp, "<table border=0 cellspacing=0 cellpadding=0>");

  fprintf (out_fp,
	   "<tr><td class=ltitle>Month</td> <td class=ltitle>SE Crawler hits</td> <td class=ltitle># keywords</td> <td class=ltitle># phrases</td><td class=ltitle># pages</td></tr>");

  m = monthlydatahead;

  while (m != NULL)
    {
      fprintf (out_fp, "<tr><td class=ltitle>%d %d</td>", m->year, m->month);
      fprintf (out_fp,
	       "<td align=right class=lrow><a href=\"botvisits_%04d_%02d.html\">%d</a></td>",
	       m->year, m->month, m->crawlerhits);
      fprintf (out_fp,
	       "<td align=right class=lrow><a href=\"keywords_%04d_%02d.html\">%d</a></td>",
	       m->year, m->month, m->keywordcount);
      fprintf (out_fp,
	       "<td align=right class=lrow><a href=\"searchstrings_%04d_%02d.html\">%d</a></td>",
	       m->year, m->month, m->phrasecount);
      fprintf (out_fp,
	       "<td align=right class=lrow><a href=\"pages_%04d_%02d.html\">%d</a></td>",
	       m->year, m->month, m->pagescount);

      fprintf (out_fp, "</tr>");
      m = m->next;
    }


  fprintf (out_fp, "</TABLE>\n<P>\n");
  page_footer ();
  fclose (out_fp);
}
