#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>

#include <sys/socket.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

#ifdef SUNOS
#define NOBLOCK O_NDELAY
#else
#define NOBLOCK O_NONBLOCK
#endif

#include "pagerank_calc.h"

unsigned long int
resolve (char *hostname)
{
  struct hostent *hp;
  unsigned long int ip;

  if (hostname == NULL)
    return INADDR_NONE;
  if (*hostname == 0)
    return INADDR_NONE;

  ip = inet_addr (hostname);
  if (ip != INADDR_NONE)
    return ip;

  if ((hp = gethostbyname (hostname)) == NULL)
    return INADDR_NONE;

  memcpy (&ip, hp->h_addr, hp->h_length);
  return ip;
}

int
sockgets (int socket, char *buf, int len)
{
  while (--len)
    {
      if (read (socket, buf, 1) != 1)
	return -1;
      if (*buf == '\n')
	break;
      if (*buf != '\r')		/* remove all CRs */
	buf++;
    }
  *buf = 0;
  return 0;
}


#define TOOLBARQUERIES_HOST "toolbarqueries.google.com"

int
getpagerank (char *url)
{
  int client_fd;
  unsigned long int addr;
  struct sockaddr_in sin_addr;
  char request[0x200];
  char *buffer;
  int len;
  int rc;
  int firstline, body;
  int pagerank;

  client_fd = socket (AF_INET, SOCK_STREAM, 0);
  if (client_fd == -1)
    return -1;


  addr = resolve (TOOLBARQUERIES_HOST);

  if (addr == INADDR_NONE)
    {
      fprintf (stderr, "Cannot resolve address "  TOOLBARQUERIES_HOST "\n");
      return -1;
    }

  memset (&sin_addr, 0, sizeof (sin_addr));
  sin_addr.sin_family = AF_INET;
  memcpy (&sin_addr.sin_addr, &addr, sizeof (addr));
  sin_addr.sin_port = htons (80);

  pagerank = -1;

  /* Connect to the server */
  if (connect
      (client_fd, (struct sockaddr *) &sin_addr,
       sizeof (struct sockaddr_in)) < 0)
    {
      printf ("pagerank: Connect failed\n");
      return -1;
    }

  sprintf (request,
	   "GET /search?client=navclient-auto&features=Rank:&q=info:%s&ch=%s HTTP/1.0\r\n",
	   url, pagerank_calchash (url));
  strcat (request, "Host: toolbarqueries.google.com\r\n");
  strcat (request, "Connection: close\r\n");
  strcat (request, "User-agent: SEOlizer 0.2\r\n");
  strcat (request, "\r\n");

  if (write (client_fd, request, strlen (request)) != strlen (request))
    {
      perror ("Write");
      return -1;
    }

  buffer = (char *) malloc (10240);
  len = 0;
  firstline = 1;
  body = 0;

  while (1)
    {
      rc = sockgets (client_fd, buffer, 10240);
      if (rc != 0)
	{
	  break;
	}

      if (buffer[strlen (buffer) - 1] == '\n')
	buffer[strlen (buffer) - 1] = 0;

      if (body)
	{
	  if (strlen (buffer) > 9)
	    {
	      if (!memcmp (buffer, "Rank_1:1:", 9))
		{
		  pagerank = atoi (buffer + 9);
		}
	    }
	}
      else
	/* process string */
      if (firstline)
	{
	  firstline = 0;
	  //puts(buffer);
	}
      else
	{
	  if (buffer[0] == 0)
	    body = 1;
	  //puts(buffer);
	}
    }				/* while(1) */


  close (client_fd);

  return pagerank;
}
