/******************************************************************************
Filename     : pagerank.c
Description  : Google PageRank Checksum Algorithm (Toolbar 3.0.x/4.0.x)
Author       : http://pagerank.gamesaga.net/
Log          : Ver 0.1 2005-09-13  first release
               Ver 1.0 2005-10-19  fixed :final character bug
               Ver 1.1 2006-10-05  refine code
******************************************************************************/

#include <stdio.h>
#include "pagerank_calc.h"

int StrToInt(char *pStr, int Init, int Factor)
{
    while (*pStr) {
        Init *= Factor;
        Init += *pStr++;
    }
    return Init;
}

int HashURL(char *pStr)
{
    unsigned int C1, C2, T1, T2;

    C1 = StrToInt(pStr, 0x1505, 0x21);
    C2 = StrToInt(pStr, 0, 0x1003F);
    C1 >>= 2;
    C1 = ((C1 >> 4) & 0x3FFFFC0) | (C1 & 0x3F);
    C1 = ((C1 >> 4) & 0x3FFC00) | (C1 & 0x3FF);
    C1 = ((C1 >> 4) & 0x3C000) | (C1 & 0x3FFF);

    T1 = (C1 & 0x3C0) << 4;
    T1 |= C1 & 0x3C;
    T1 = (T1 << 2) | (C2 & 0xF0F);

    T2 = (C1 & 0xFFFFC000) << 4;
    T2 |= C1 & 0x3C00;
    T2 = (T2 << 0xA) | (C2 & 0xF0F0000);

    return (T1 | T2);
}

char CheckHash(unsigned int HashInt)
{
    int Check = 0, Flag = 0;
    int Remainder;

    do {
        Remainder = HashInt % 10;
        HashInt /= 10;
        if (1 == (Flag % 2) ){
            Remainder += Remainder;
            Remainder = (Remainder / 10) + (Remainder % 10);
        }
        Check += Remainder;
        Flag ++;
    } while( 0 != HashInt);

    Check %= 10;
    if (0 != Check) {
        Check = 10 - Check;
        if (1 == (Flag % 2)) {
            if (1 == (Check % 2)) {
                Check += 9;
            }
            Check >>= 1;
        }
    }
    Check += 0x30;
    return Check;
}

static char _pagerank_hash_buf[0x100];

char *pagerank_calchash(char *url)
{
    unsigned int HashInt;

    HashInt = HashURL(url);
    sprintf(_pagerank_hash_buf, "7%c%u", CheckHash(HashInt), HashInt);
    return _pagerank_hash_buf;
}


#ifdef __TEST__

int main(int argc, char* argv[])
{
    unsigned int HashInt;

    printf("Google PageRank Checksum Calculator V1.2\n");
    if (argc < 2) {
        printf("Usage: %s [URL]\n",argv[0]);
        return 1;
    }

    HashInt = HashURL(argv[1]);
    printf("Checksum=7%c%u\n", CheckHash(HashInt), HashInt);
    return 0;
}

#endif
